# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/app_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from union.internal.identity import app_payload_pb2 as identity_dot_app__payload__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aidentity/app_service.proto\x12\x11\x63loudidl.identity\x1a\x1cgoogle/api/annotations.proto\x1a\x1aidentity/app_payload.proto2\xda\x04\n\x0b\x41ppsService\x12q\n\x06\x43reate\x12#.cloudidl.identity.CreateAppRequest\x1a$.cloudidl.identity.CreateAppResponse\"\x1c\x82\xd3\xe4\x93\x02\x16\"\x11/oauth_app/api/v1:\x01*\x12q\n\x03Get\x12 .cloudidl.identity.GetAppRequest\x1a!.cloudidl.identity.GetAppResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/oauth_app/api/v1/{client_id}\x12}\n\x06Update\x12#.cloudidl.identity.UpdateAppRequest\x1a$.cloudidl.identity.UpdateAppResponse\"(\x82\xd3\xe4\x93\x02\"\x1a\x1d/oauth_app/api/v1/{client_id}:\x01*\x12z\n\x06\x44\x65lete\x12#.cloudidl.identity.DeleteAppRequest\x1a$.cloudidl.identity.DeleteAppResponse\"%\x82\xd3\xe4\x93\x02\x1f*\x1d/oauth_app/api/v1/{client_id}\x12j\n\x04List\x12\".cloudidl.identity.ListAppsRequest\x1a#.cloudidl.identity.ListAppsResponse\"\x19\x82\xd3\xe4\x93\x02\x13\x12\x11/oauth_app/api/v1B\xbc\x01\n\x15\x63om.cloudidl.identityB\x0f\x41ppServiceProtoH\x02P\x01Z+github.com/unionai/cloud/gen/pb-go/identity\xa2\x02\x03\x43IX\xaa\x02\x11\x43loudidl.Identity\xca\x02\x11\x43loudidl\\Identity\xe2\x02\x1d\x43loudidl\\Identity\\GPBMetadata\xea\x02\x12\x43loudidl::Identityb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity.app_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.cloudidl.identityB\017AppServiceProtoH\002P\001Z+github.com/unionai/cloud/gen/pb-go/identity\242\002\003CIX\252\002\021Cloudidl.Identity\312\002\021Cloudidl\\Identity\342\002\035Cloudidl\\Identity\\GPBMetadata\352\002\022Cloudidl::Identity'
  _APPSSERVICE.methods_by_name['Create']._options = None
  _APPSSERVICE.methods_by_name['Create']._serialized_options = b'\202\323\344\223\002\026\"\021/oauth_app/api/v1:\001*'
  _APPSSERVICE.methods_by_name['Get']._options = None
  _APPSSERVICE.methods_by_name['Get']._serialized_options = b'\202\323\344\223\002\037\022\035/oauth_app/api/v1/{client_id}'
  _APPSSERVICE.methods_by_name['Update']._options = None
  _APPSSERVICE.methods_by_name['Update']._serialized_options = b'\202\323\344\223\002\"\032\035/oauth_app/api/v1/{client_id}:\001*'
  _APPSSERVICE.methods_by_name['Delete']._options = None
  _APPSSERVICE.methods_by_name['Delete']._serialized_options = b'\202\323\344\223\002\037*\035/oauth_app/api/v1/{client_id}'
  _APPSSERVICE.methods_by_name['List']._options = None
  _APPSSERVICE.methods_by_name['List']._serialized_options = b'\202\323\344\223\002\023\022\021/oauth_app/api/v1'
  _globals['_APPSSERVICE']._serialized_start=108
  _globals['_APPSSERVICE']._serialized_end=710
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from union.internal.identity import app_payload_pb2 as identity_dot_app__payload__pb2


class AppsServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/cloudidl.identity.AppsService/Create',
                request_serializer=identity_dot_app__payload__pb2.CreateAppRequest.SerializeToString,
                response_deserializer=identity_dot_app__payload__pb2.CreateAppResponse.FromString,
                )
        self.Get = channel.unary_unary(
                '/cloudidl.identity.AppsService/Get',
                request_serializer=identity_dot_app__payload__pb2.GetAppRequest.SerializeToString,
                response_deserializer=identity_dot_app__payload__pb2.GetAppResponse.FromString,
                )
        self.Update = channel.unary_unary(
                '/cloudidl.identity.AppsService/Update',
                request_serializer=identity_dot_app__payload__pb2.UpdateAppRequest.SerializeToString,
                response_deserializer=identity_dot_app__payload__pb2.UpdateAppResponse.FromString,
                )
        self.Delete = channel.unary_unary(
                '/cloudidl.identity.AppsService/Delete',
                request_serializer=identity_dot_app__payload__pb2.DeleteAppRequest.SerializeToString,
                response_deserializer=identity_dot_app__payload__pb2.DeleteAppResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/cloudidl.identity.AppsService/List',
                request_serializer=identity_dot_app__payload__pb2.ListAppsRequest.SerializeToString,
                response_deserializer=identity_dot_app__payload__pb2.ListAppsResponse.FromString,
                )


class AppsServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Get(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AppsServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=identity_dot_app__payload__pb2.CreateAppRequest.FromString,
                    response_serializer=identity_dot_app__payload__pb2.CreateAppResponse.SerializeToString,
            ),
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=identity_dot_app__payload__pb2.GetAppRequest.FromString,
                    response_serializer=identity_dot_app__payload__pb2.GetAppResponse.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=identity_dot_app__payload__pb2.UpdateAppRequest.FromString,
                    response_serializer=identity_dot_app__payload__pb2.UpdateAppResponse.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=identity_dot_app__payload__pb2.DeleteAppRequest.FromString,
                    response_serializer=identity_dot_app__payload__pb2.DeleteAppResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=identity_dot_app__payload__pb2.ListAppsRequest.FromString,
                    response_serializer=identity_dot_app__payload__pb2.ListAppsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cloudidl.identity.AppsService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AppsService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.AppsService/Create',
            identity_dot_app__payload__pb2.CreateAppRequest.SerializeToString,
            identity_dot_app__payload__pb2.CreateAppResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.AppsService/Get',
            identity_dot_app__payload__pb2.GetAppRequest.SerializeToString,
            identity_dot_app__payload__pb2.GetAppResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.AppsService/Update',
            identity_dot_app__payload__pb2.UpdateAppRequest.SerializeToString,
            identity_dot_app__payload__pb2.UpdateAppResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.AppsService/Delete',
            identity_dot_app__payload__pb2.DeleteAppRequest.SerializeToString,
            identity_dot_app__payload__pb2.DeleteAppResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.AppsService/List',
            identity_dot_app__payload__pb2.ListAppsRequest.SerializeToString,
            identity_dot_app__payload__pb2.ListAppsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/app_payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import list_pb2 as common_dot_list__pb2
from union.internal.identity import app_definition_pb2 as identity_dot_app__definition__pb2
from union.internal.identity import enums_pb2 as identity_dot_enums__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aidentity/app_payload.proto\x12\x11\x63loudidl.identity\x1a\x11\x63ommon/list.proto\x1a\x1didentity/app_definition.proto\x1a\x14identity/enums.proto\x1a\x17validate/validate.proto\"\xd5\x05\n\x10\x43reateAppRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x1b\n\tclient_id\x18\x02 \x01(\tR\x08\x63lientId\x12#\n\rredirect_uris\x18\x04 \x03(\tR\x0credirectUris\x12G\n\x0eresponse_types\x18\x05 \x03(\x0e\x32 .cloudidl.identity.ResponseTypesR\rresponseTypes\x12>\n\x0bgrant_types\x18\x06 \x03(\x0e\x32\x1d.cloudidl.identity.GrantTypesR\ngrantTypes\x12M\n\x10\x61pplication_type\x18\x07 \x01(\x0e\x32\".cloudidl.identity.ApplicationTypeR\x0f\x61pplicationType\x12\x1a\n\x08\x63ontacts\x18\x08 \x03(\tR\x08\x63ontacts\x12(\n\x0b\x63lient_name\x18\t \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\nclientName\x12\x19\n\x08logo_uri\x18\n \x01(\tR\x07logoUri\x12\x1d\n\nclient_uri\x18\x0b \x01(\tR\tclientUri\x12\x1d\n\npolicy_uri\x18\x0c \x01(\tR\tpolicyUri\x12\x17\n\x07tos_uri\x18\r \x01(\tR\x06tosUri\x12\x19\n\x08jwks_uri\x18\x0e \x01(\tR\x07jwksUri\x12g\n\x1atoken_endpoint_auth_method\x18\x0f \x01(\x0e\x32*.cloudidl.identity.TokenEndpointAuthMethodR\x17tokenEndpointAuthMethod\x12G\n\x0e\x63onsent_method\x18\x10 \x01(\x0e\x32 .cloudidl.identity.ConsentMethodR\rconsentMethod\"=\n\x11\x43reateAppResponse\x12(\n\x03\x61pp\x18\x01 \x01(\x0b\x32\x16.cloudidl.identity.AppR\x03\x61pp\"Y\n\rGetAppRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12$\n\tclient_id\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x08\x63lientId\":\n\x0eGetAppResponse\x12(\n\x03\x61pp\x18\x01 \x01(\x0b\x32\x16.cloudidl.identity.AppR\x03\x61pp\"\x8c\x05\n\x10UpdateAppRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12$\n\tclient_id\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x08\x63lientId\x12#\n\rredirect_uris\x18\x04 \x03(\tR\x0credirectUris\x12G\n\x0eresponse_types\x18\x05 \x03(\x0e\x32 .cloudidl.identity.ResponseTypesR\rresponseTypes\x12>\n\x0bgrant_types\x18\x06 \x03(\x0e\x32\x1d.cloudidl.identity.GrantTypesR\ngrantTypes\x12M\n\x10\x61pplication_type\x18\x07 \x01(\x0e\x32\".cloudidl.identity.ApplicationTypeR\x0f\x61pplicationType\x12\x1a\n\x08\x63ontacts\x18\x08 \x03(\tR\x08\x63ontacts\x12\x1f\n\x0b\x63lient_name\x18\t \x01(\tR\nclientName\x12\x19\n\x08logo_uri\x18\n \x01(\tR\x07logoUri\x12\x1d\n\nclient_uri\x18\x0b \x01(\tR\tclientUri\x12\x1d\n\npolicy_uri\x18\x0c \x01(\tR\tpolicyUri\x12\x17\n\x07tos_uri\x18\r \x01(\tR\x06tosUri\x12\x19\n\x08jwks_uri\x18\x0e \x01(\tR\x07jwksUri\x12g\n\x1atoken_endpoint_auth_method\x18\x0f \x01(\x0e\x32*.cloudidl.identity.TokenEndpointAuthMethodR\x17tokenEndpointAuthMethod\"=\n\x11UpdateAppResponse\x12(\n\x03\x61pp\x18\x01 \x01(\x0b\x32\x16.cloudidl.identity.AppR\x03\x61pp\"\\\n\x10\x44\x65leteAppRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12$\n\tclient_id\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x08\x63lientId\"\x13\n\x11\x44\x65leteAppResponse\"m\n\x0fListAppsRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x36\n\x07request\x18\x02 \x01(\x0b\x32\x1c.cloudidl.common.ListRequestR\x07request\"T\n\x10ListAppsResponse\x12*\n\x04\x61pps\x18\x01 \x03(\x0b\x32\x16.cloudidl.identity.AppR\x04\x61pps\x12\x14\n\x05token\x18\x02 \x01(\tR\x05tokenB\xbc\x01\n\x15\x63om.cloudidl.identityB\x0f\x41ppPayloadProtoH\x02P\x01Z+github.com/unionai/cloud/gen/pb-go/identity\xa2\x02\x03\x43IX\xaa\x02\x11\x43loudidl.Identity\xca\x02\x11\x43loudidl\\Identity\xe2\x02\x1d\x43loudidl\\Identity\\GPBMetadata\xea\x02\x12\x43loudidl::Identityb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity.app_payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.cloudidl.identityB\017AppPayloadProtoH\002P\001Z+github.com/unionai/cloud/gen/pb-go/identity\242\002\003CIX\252\002\021Cloudidl.Identity\312\002\021Cloudidl\\Identity\342\002\035Cloudidl\\Identity\\GPBMetadata\352\002\022Cloudidl::Identity'
  _CREATEAPPREQUEST.fields_by_name['client_name']._options = None
  _CREATEAPPREQUEST.fields_by_name['client_name']._serialized_options = b'\372B\004r\002\020\001'
  _GETAPPREQUEST.fields_by_name['client_id']._options = None
  _GETAPPREQUEST.fields_by_name['client_id']._serialized_options = b'\372B\004r\002\020\001'
  _UPDATEAPPREQUEST.fields_by_name['client_id']._options = None
  _UPDATEAPPREQUEST.fields_by_name['client_id']._serialized_options = b'\372B\004r\002\020\001'
  _DELETEAPPREQUEST.fields_by_name['client_id']._options = None
  _DELETEAPPREQUEST.fields_by_name['client_id']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CREATEAPPREQUEST']._serialized_start=147
  _globals['_CREATEAPPREQUEST']._serialized_end=872
  _globals['_CREATEAPPRESPONSE']._serialized_start=874
  _globals['_CREATEAPPRESPONSE']._serialized_end=935
  _globals['_GETAPPREQUEST']._serialized_start=937
  _globals['_GETAPPREQUEST']._serialized_end=1026
  _globals['_GETAPPRESPONSE']._serialized_start=1028
  _globals['_GETAPPRESPONSE']._serialized_end=1086
  _globals['_UPDATEAPPREQUEST']._serialized_start=1089
  _globals['_UPDATEAPPREQUEST']._serialized_end=1741
  _globals['_UPDATEAPPRESPONSE']._serialized_start=1743
  _globals['_UPDATEAPPRESPONSE']._serialized_end=1804
  _globals['_DELETEAPPREQUEST']._serialized_start=1806
  _globals['_DELETEAPPREQUEST']._serialized_end=1898
  _globals['_DELETEAPPRESPONSE']._serialized_start=1900
  _globals['_DELETEAPPRESPONSE']._serialized_end=1919
  _globals['_LISTAPPSREQUEST']._serialized_start=1921
  _globals['_LISTAPPSREQUEST']._serialized_end=2030
  _globals['_LISTAPPSRESPONSE']._serialized_start=2032
  _globals['_LISTAPPSRESPONSE']._serialized_end=2116
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from union.internal.app import app_payload_pb2 as app_dot_app__payload__pb2


class AppServiceStub(object):
    """AppService defines the service for managing apps.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/cloudidl.app.AppService/Create',
                request_serializer=app_dot_app__payload__pb2.CreateRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.CreateResponse.FromString,
                )
        self.Get = channel.unary_unary(
                '/cloudidl.app.AppService/Get',
                request_serializer=app_dot_app__payload__pb2.GetRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.GetResponse.FromString,
                )
        self.Update = channel.unary_unary(
                '/cloudidl.app.AppService/Update',
                request_serializer=app_dot_app__payload__pb2.UpdateRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.UpdateResponse.FromString,
                )
        self.UpdateStatus = channel.unary_unary(
                '/cloudidl.app.AppService/UpdateStatus',
                request_serializer=app_dot_app__payload__pb2.UpdateStatusRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.UpdateStatusResponse.FromString,
                )
        self.Delete = channel.unary_unary(
                '/cloudidl.app.AppService/Delete',
                request_serializer=app_dot_app__payload__pb2.DeleteRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.DeleteResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/cloudidl.app.AppService/List',
                request_serializer=app_dot_app__payload__pb2.ListRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.ListResponse.FromString,
                )
        self.Watch = channel.unary_stream(
                '/cloudidl.app.AppService/Watch',
                request_serializer=app_dot_app__payload__pb2.WatchRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.WatchResponse.FromString,
                )
        self.Lease = channel.unary_stream(
                '/cloudidl.app.AppService/Lease',
                request_serializer=app_dot_app__payload__pb2.LeaseRequest.SerializeToString,
                response_deserializer=app_dot_app__payload__pb2.LeaseResponse.FromString,
                )


class AppServiceServicer(object):
    """AppService defines the service for managing apps.
    """

    def Create(self, request, context):
        """Create creates a new app.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Get(self, request, context):
        """Get retrieves an app by its identifier.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Update updates an existing app.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateStatus(self, request, context):
        """UpdateStatus updates the status of an existing app.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Delete deletes an app by its identifier.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """List lists all apps with optional filtering.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Watch(self, request, context):
        """Watch watches for app events.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Lease(self, request, context):
        """Lease leases apps.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AppServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=app_dot_app__payload__pb2.CreateRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.CreateResponse.SerializeToString,
            ),
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=app_dot_app__payload__pb2.GetRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.GetResponse.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=app_dot_app__payload__pb2.UpdateRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.UpdateResponse.SerializeToString,
            ),
            'UpdateStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateStatus,
                    request_deserializer=app_dot_app__payload__pb2.UpdateStatusRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.UpdateStatusResponse.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=app_dot_app__payload__pb2.DeleteRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.DeleteResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=app_dot_app__payload__pb2.ListRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.ListResponse.SerializeToString,
            ),
            'Watch': grpc.unary_stream_rpc_method_handler(
                    servicer.Watch,
                    request_deserializer=app_dot_app__payload__pb2.WatchRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.WatchResponse.SerializeToString,
            ),
            'Lease': grpc.unary_stream_rpc_method_handler(
                    servicer.Lease,
                    request_deserializer=app_dot_app__payload__pb2.LeaseRequest.FromString,
                    response_serializer=app_dot_app__payload__pb2.LeaseResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cloudidl.app.AppService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AppService(object):
    """AppService defines the service for managing apps.
    """

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.app.AppService/Create',
            app_dot_app__payload__pb2.CreateRequest.SerializeToString,
            app_dot_app__payload__pb2.CreateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.app.AppService/Get',
            app_dot_app__payload__pb2.GetRequest.SerializeToString,
            app_dot_app__payload__pb2.GetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.app.AppService/Update',
            app_dot_app__payload__pb2.UpdateRequest.SerializeToString,
            app_dot_app__payload__pb2.UpdateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.app.AppService/UpdateStatus',
            app_dot_app__payload__pb2.UpdateStatusRequest.SerializeToString,
            app_dot_app__payload__pb2.UpdateStatusResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.app.AppService/Delete',
            app_dot_app__payload__pb2.DeleteRequest.SerializeToString,
            app_dot_app__payload__pb2.DeleteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.app.AppService/List',
            app_dot_app__payload__pb2.ListRequest.SerializeToString,
            app_dot_app__payload__pb2.ListResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Watch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/cloudidl.app.AppService/Watch',
            app_dot_app__payload__pb2.WatchRequest.SerializeToString,
            app_dot_app__payload__pb2.WatchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Lease(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/cloudidl.app.AppService/Lease',
            app_dot_app__payload__pb2.LeaseRequest.SerializeToString,
            app_dot_app__payload__pb2.LeaseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from union.internal.identity import enums_pb2 as _enums_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class App(_message.Message):
    __slots__ = ["organization", "client_id", "client_id_issued_at", "redirect_uris", "response_types", "grant_types", "application_type", "contacts", "client_name", "logo_uri", "client_uri", "policy_uri", "tos_uri", "jwks_uri", "token_endpoint_auth_method", "client_secret", "client_secret_expires_at", "profile"]
    ORGANIZATION_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ID_ISSUED_AT_FIELD_NUMBER: _ClassVar[int]
    REDIRECT_URIS_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_TYPES_FIELD_NUMBER: _ClassVar[int]
    GRANT_TYPES_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_TYPE_FIELD_NUMBER: _ClassVar[int]
    CONTACTS_FIELD_NUMBER: _ClassVar[int]
    CLIENT_NAME_FIELD_NUMBER: _ClassVar[int]
    LOGO_URI_FIELD_NUMBER: _ClassVar[int]
    CLIENT_URI_FIELD_NUMBER: _ClassVar[int]
    POLICY_URI_FIELD_NUMBER: _ClassVar[int]
    TOS_URI_FIELD_NUMBER: _ClassVar[int]
    JWKS_URI_FIELD_NUMBER: _ClassVar[int]
    TOKEN_ENDPOINT_AUTH_METHOD_FIELD_NUMBER: _ClassVar[int]
    CLIENT_SECRET_FIELD_NUMBER: _ClassVar[int]
    CLIENT_SECRET_EXPIRES_AT_FIELD_NUMBER: _ClassVar[int]
    PROFILE_FIELD_NUMBER: _ClassVar[int]
    organization: str
    client_id: str
    client_id_issued_at: _timestamp_pb2.Timestamp
    redirect_uris: _containers.RepeatedScalarFieldContainer[str]
    response_types: _containers.RepeatedScalarFieldContainer[_enums_pb2.ResponseTypes]
    grant_types: _containers.RepeatedScalarFieldContainer[_enums_pb2.GrantTypes]
    application_type: _enums_pb2.ApplicationType
    contacts: _containers.RepeatedScalarFieldContainer[str]
    client_name: str
    logo_uri: str
    client_uri: str
    policy_uri: str
    tos_uri: str
    jwks_uri: str
    token_endpoint_auth_method: _enums_pb2.TokenEndpointAuthMethod
    client_secret: str
    client_secret_expires_at: _timestamp_pb2.Timestamp
    profile: Profile
    def __init__(self, organization: _Optional[str] = ..., client_id: _Optional[str] = ..., client_id_issued_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., redirect_uris: _Optional[_Iterable[str]] = ..., response_types: _Optional[_Iterable[_Union[_enums_pb2.ResponseTypes, str]]] = ..., grant_types: _Optional[_Iterable[_Union[_enums_pb2.GrantTypes, str]]] = ..., application_type: _Optional[_Union[_enums_pb2.ApplicationType, str]] = ..., contacts: _Optional[_Iterable[str]] = ..., client_name: _Optional[str] = ..., logo_uri: _Optional[str] = ..., client_uri: _Optional[str] = ..., policy_uri: _Optional[str] = ..., tos_uri: _Optional[str] = ..., jwks_uri: _Optional[str] = ..., token_endpoint_auth_method: _Optional[_Union[_enums_pb2.TokenEndpointAuthMethod, str]] = ..., client_secret: _Optional[str] = ..., client_secret_expires_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., profile: _Optional[_Union[Profile, _Mapping]] = ...) -> None: ...

class Profile(_message.Message):
    __slots__ = ["apporg"]
    APPORG_FIELD_NUMBER: _ClassVar[int]
    apporg: str
    def __init__(self, apporg: _Optional[str] = ...) -> None: ...



from datetime import timedelta
from typing import Any, Dict, Optional, Type, Union

from flyteidl.core import interface_pb2
from flytekit.core.artifact import Artifact, ArtifactQuery, Partition, Serializer, TimePartition
from flytekit.core.context_manager import FlyteContextManager
from flytekit.core.launch_plan import LaunchPlan
from flytekit.core.schedule import LaunchPlanTriggerBase
from flytekit.core.type_engine import TypeEngine
from flytekit.models.interface import ParameterMap, Variable

from union._vendor.isodate import duration_isoformat
from union.internal.artifacts import artifacts_pb2


class OnArtifact(LaunchPlanTriggerBase):
    """
    Event used to link upstream and downstream workflows together.

    :param trigger_on: Artifact on which to trigger.
    :param inputs: Dict of inputs.

    Example usage::

        OnArtifact(
            trigger_on=dailyArtifact,
            inputs={
                # Use the matched Artifact
                "today_upstream": dailyArtifact,
                "yesterday_upstream": dailyArtifact.query(
                    time_partition=dailyArtifact.
                    time_partition - timedelta(days=1)),
                # Use the matched hourly Artifact
                "other_daily_upstream": hourlyArtifact.query(
                    partitions={"region": "LAX"}),
                # Static value "SEA" that will be passed as input
                "region": "SEA",
                "other_artifact": UnrelatedArtifact.query(
                    time_partition=dailyArtifact.
                    time_partition - timedelta(days=1)),
                "other_artifact_2": UnrelatedArtifact.query(
                    time_partition=hourlyArtifact.time_partition.truncate_to_day()),
                "other_artifact_3": UnrelatedArtifact.query(
                    region=hourlyArtifact.time_partition.truncate_to_day()),
            },
        )
    """

    def __init__(
        self,
        trigger_on: Union[Artifact, ArtifactQuery],
        inputs: Optional[Dict[str, Union[Any, Artifact, ArtifactQuery]]] = None,
    ):
        super().__init__()
        if not isinstance(trigger_on, Artifact) and not isinstance(trigger_on, ArtifactQuery):
            raise ValueError("Trigger must be called with a single artifact or query")
        self.trigger_on = trigger_on
        self.inputs = inputs or {}  # user doesn't need to specify if the launch plan doesn't take inputs
        for k, v in self.inputs.items():
            if isinstance(v, ArtifactQuery):
                # This one binding reflects all the bindings across all the partition values as well.
                if v.binding:
                    if v.binding != self.trigger_on:
                        raise ValueError(
                            f"Binding {v.binding} id {id(v.binding)} must"
                            f"reference the triggering artifact {self.trigger_on}"
                        )

            # If the input map of a trigger is an artifact instance, union takes that to mean the instance of the
            # artifact that kicks off the trigger. So there can only be one. Otherwise, use a query.
            if isinstance(v, Artifact):
                if isinstance(self.trigger_on, Artifact) and v is not self.trigger_on:
                    raise ValueError(f"Binding {v} id {id(v)} must reference triggering artifact {self.trigger_on}")
                if isinstance(self.trigger_on, ArtifactQuery) and v is not self.trigger_on.artifact:
                    raise ValueError(
                        f"Binding {v} id {id(v)} must reference triggering artifact {self.trigger_on.artifact}"
                    )

    def get_parameter_map(
        self, input_python_interface: Dict[str, Type], input_typed_interface: Dict[str, Variable]
    ) -> interface_pb2.ParameterMap:
        """
        This is the key function that enables triggers to work. When declaring a trigger, the user specifies an input
        map in the form of artifacts, artifact time partitions, and artifact queries (possibly on unrelated artifacts).
        When it comes time to create the trigger, we need to convert all of these into a parameter map (because we've
        chosen Parameter as the method by which things like artifact queries are passed around). This function does
        that, and converts constants to Literals.
        """
        ctx = FlyteContextManager.current_context()
        pm = {}
        for k, v in self.inputs.items():
            var = input_typed_interface[k].to_flyte_idl()
            if isinstance(v, Artifact):
                aq = v.embed_as_query()
                p = interface_pb2.Parameter(var=var, artifact_query=aq)
            elif isinstance(v, ArtifactQuery):
                p = interface_pb2.Parameter(var=var, artifact_query=v.to_flyte_idl(binding=self.trigger_on))
            elif isinstance(v, TimePartition):
                expr = None
                if v.other and isinstance(v.other, timedelta):
                    expr = duration_isoformat(v.other)
                aq = v.reference_artifact.embed_as_query(bind_to_time_partition=True, expr=expr, op=v.op)
                p = interface_pb2.Parameter(var=var, artifact_query=aq)
            elif isinstance(v, Partition):
                # The reason is that if we bind to arbitrary partitions, we'll have to start keeping track of types
                # and if the experiment service is written in non-python, we'd have to reimplement a type engine in
                # the other language
                raise ValueError(
                    "Don't bind to non-time partitions. Time partitions are okay because of the known type."
                )
            else:
                lit = TypeEngine.to_literal(ctx, v, input_python_interface[k], input_typed_interface[k].type)
                p = interface_pb2.Parameter(var=var, default=lit.to_flyte_idl())

            pm[k] = p
        return interface_pb2.ParameterMap(parameters=pm)

    def to_flyte_idl(self, *args, **kwargs) -> artifacts_pb2.Trigger:
        lp: LaunchPlan = args[0]
        # project/domain will be empty - to be bound later at registration time.
        if isinstance(self.trigger_on, Artifact):
            # Handle the case where we trigger just on the artifact. As long as the partition keys match, any instance
            # of the artifact will trigger.
            artifact_id = self.trigger_on.to_id_idl()
        else:
            # Handle the case where we trigger on a query. In this case both the partition keys and values must match.
            aq = Serializer.artifact_query_to_idl(self.trigger_on, strict=True)
            if not aq.HasField("artifact_id"):
                raise ValueError(f"The query arg to trigger on must needs to produce an artifact id, got {aq}")
            artifact_id = aq.artifact_id

        pm = self.get_parameter_map(lp.python_interface.inputs, lp.interface.inputs)
        pm_model = ParameterMap.from_flyte_idl(pm)

        return artifacts_pb2.Trigger(
            trigger=artifact_id,
            trigger_inputs=pm_model.to_flyte_idl(),
        )



"""Module to manage applications on Union."""

import logging
import os
import tarfile
from copy import deepcopy
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import List, Optional

import grpc
from flyteidl.core.tasks_pb2 import Resources
from flytekit.clients.auth_helper import get_authenticated_channel
from flytekit.configuration import Config
from flytekit.core.artifact import ArtifactQuery
from flytekit.models.core.types import BlobType
from rich.console import Console

from union._config import (
    ConfigSource,
    ConfigWithSource,
    _get_config_obj,
)
from union.app import App
from union.app._models import AppSerializationSettings, Input, MaterializedInput, URLQuery
from union.internal.app.app_definition_pb2 import App as AppIDL
from union.internal.app.app_definition_pb2 import Identifier, Spec, Status
from union.internal.app.app_payload_pb2 import (
    CreateRequest,
    CreateResponse,
    GetRequest,
    GetResponse,
    ListResponse,
    UpdateRequest,
    UpdateResponse,
    WatchRequest,
    WatchResponse,
)
from union.internal.app.app_payload_pb2 import (
    ListRequest as ListAppsRequest,
)
from union.internal.app.app_service_pb2_grpc import AppServiceStub
from union.internal.common.identifier_pb2 import ProjectIdentifier
from union.internal.common.list_pb2 import ListRequest
from union.remote._remote import UnionRemote

logger = logging.getLogger(__name__)

FILES_TAR_FILE_NAME = "include-files.tar.gz"


class AppRemote:
    def __init__(
        self,
        project: str,
        domain: str,
        config: Optional[Config] = None,
    ):
        if config is None:
            config = _get_config_obj(config, default_to_union_semantics=True)
        else:
            config_with_source = ConfigWithSource(config=config, source=ConfigSource.REMOTE)
            config = _get_config_obj(config_with_source, default_to_union_semantics=True)

        self.config = config
        self.project = project
        self.domain = domain
        self._union_remote = UnionRemote(config=config, default_domain=domain, default_project=project)

    def list(self) -> List[AppIDL]:
        def create_list_request(token: str):
            return ListAppsRequest(
                request=ListRequest(token=token),
                org=self.org,
                project=ProjectIdentifier(name=self.project, domain=self.domain, organization=self.org),
            )

        results = []
        response: ListResponse
        token, has_next = "", True

        while has_next:
            list_request = create_list_request(token=token)

            response = self.sync_client.List(list_request)
            token = response.token
            has_next = token != ""

            results.extend(response.apps)

        return results

    def create_or_update(self, app: App):
        try:
            self.get(name=app.name)
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                self.create(app)
                return
            raise

        try:
            self.update(app)
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.ABORTED and "Either the change has already" in e.details():
                console = Console()
                console.print(f"{app.name} app was already deployed with the current spec")
                return
            raise

    def get(self, name: str) -> AppIDL:
        app_id = Identifier(org=self.org, project=self.project, domain=self.domain, name=name)
        get_app_request = GetRequest(app_id=app_id)

        get_response: GetResponse = self.sync_client.Get(get_app_request)
        app = get_response.app

        return app

    def update(self, app: App):
        additional_distribution = self.upload_files(app)
        materialized_input_values = self.materialize_values(app)

        settings = AppSerializationSettings(
            org=self.org,
            project=self.project,
            domain=self.domain,
            additional_distribution=additional_distribution,
            desired_state=Spec.DesiredState.DESIRED_STATE_STARTED,
            materialized_inputs=materialized_input_values,
        )
        new_app_idl = app._to_union_idl(settings=settings)

        get_app_request = GetRequest(app_id=new_app_idl.metadata.id)
        get_response: GetResponse = self.sync_client.Get(get_app_request)

        old_app_idl = get_response.app

        updated_app_idl = AppIDL(
            metadata=old_app_idl.metadata,
            spec=new_app_idl.spec,
            status=old_app_idl.status,
        )

        console = Console()
        update_request = UpdateRequest(app=updated_app_idl)

        _url = old_app_idl.status.ingress.public_url
        url = f"[link={_url}]{_url}[/link]"
        console.print(f"✨ Updating Application: {app.name} with endpoint: {url}")
        update_response: UpdateResponse = self.sync_client.Update(update_request)

        updated_app = update_response.app

        watch_request = WatchRequest(app_id=updated_app.metadata.id)

        def watch():
            response: WatchResponse
            with console.status("✨ Updating Application") as status:
                for response in self.sync_client.Watch(watch_request):
                    status = response.create_event.app.status.conditions[-1].deployment_status
                    if status in (
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_UNSPECIFIED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_STOPPED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_FAILED,
                    ):
                        raise RuntimeError("Application failed to update")
                    elif status in (
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_UNASSIGNED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_ASSIGNED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_PENDING,
                    ):
                        status.update("✨ Updating Application...")
                    elif status == Status.DeploymentStatus.DEPLOYMENT_STATUS_STARTED:
                        status.update("🚀 Application updated!")
                        return response.app_id

        # TODO: Fix when watch works
        # watch()
        # console.print(f"Updated endpoint at: {url}")

    def create(self, app: App):
        additional_distribution = self.upload_files(app)
        materialized_input_values = self.materialize_values(app)

        settings = AppSerializationSettings(
            org=self.org,
            project=self.project,
            domain=self.domain,
            additional_distribution=additional_distribution,
            desired_state=Spec.DesiredState.DESIRED_STATE_STARTED,
            materialized_inputs=materialized_input_values,
        )
        app_idl = app._to_union_idl(settings=settings)

        console = Console()
        console.print(f"✨ Creating Application: {app.name}")

        create_request = CreateRequest(app=app_idl)
        create_response: CreateResponse = self.sync_client.Create(create_request)
        new_app = create_response.app

        _url = new_app.status.ingress.public_url
        url = f"[link={_url}]{_url}[/link]"

        logger.debug(f"Create Response {create_response} with endpoint: {url}")
        watch_request = WatchRequest(app_id=new_app.metadata.id)

        def watch() -> Identifier:
            with console.status("✨ Starting Application") as status:
                response: WatchResponse
                for response in self.sync_client.Watch(watch_request):
                    status = response.create_event.app.status
                    if status in (
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_UNSPECIFIED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_STOPPED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_FAILED,
                    ):
                        raise RuntimeError("Application failed to launch")
                    elif status in (
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_UNASSIGNED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_ASSIGNED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_PENDING,
                    ):
                        status.update("✨ Starting Application...")
                    elif status == Status.DeploymentStatus.DEPLOYMENT_STATUS_STARTED:
                        status.update("🚀 Application deployed!")
                        return response.app_id

        # TODO: Fix when watch works
        # watch()
        console.print(f"Created Endpoint at: {url}")

    def stop(self, name: str):
        app_idl = self.get(name)
        app_idl.spec.desired_state = Spec.DesiredState.DESIRED_STATE_STOPPED

        console = Console()
        console.print(f"⏳ Stopping Application: {name}")

        update_request = UpdateRequest(app=app_idl)
        self.sync_client.Update(update_request)

        watch_request = WatchRequest(app_id=app_idl.metadata.id)

        def watch():
            response: WatchResponse
            with console.status("⏳ Stopping Application...") as status:
                for response in self.sync_client.Watch(watch_request):
                    status = response.delete_event.app.status
                    if status in (
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_UNSPECIFIED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_UNASSIGNED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_ASSIGNED,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_PENDING,
                        Status.DeploymentStatus.DEPLOYMENT_STATUS_STARTED,
                    ):
                        status.update("⏳ Stopping Application...")
                    elif status == Status.DeploymentStatus.DEPLOYMENT_STATUS_FAILED:
                        status.update("💥 Application stopped because it failed to launch")
                        break
                    elif status == Status.DeploymentStatus.DEPLOYMENT_STATUS_STOPPED:
                        status.update("💥 Application stopped")
                        break

        # TODO: Uncomment when update actually delete apps
        # watch()

    def _watch(self, name: App):
        """This is for debugging."""
        app_idl = self.get(name)
        watch_request = WatchRequest(app_id=app_idl.metadata.id)

        console = Console()

        def watch():
            response: WatchResponse
            with console.status("⏳ Watching Application..."):
                for response in self.sync_client.Watch(watch_request):
                    console.print(response)

        # TODO: Uncomment when update actually delete apps
        watch()

    def _watch_async(self, name: App):
        app_idl = self.get(name)
        watch_request = WatchRequest(app_id=app_idl.metadata.id)

        console = Console()

        async def watch():
            response: WatchResponse
            with console.status("⏳ Watching Application..."):
                async for response in self.async_client.Watch(watch_request):
                    console.print(response)

        # TODO: Uncomment when update actually delete apps
        import asyncio

        asyncio.run(watch())

    def upload_files(self, app: App) -> Optional[str]:
        """Upload files required by app."""
        if not app.include:
            return None

        with TemporaryDirectory() as temp_dir:
            tar_path = os.path.join(temp_dir, FILES_TAR_FILE_NAME)
            with tarfile.open(tar_path, "w:gz") as tar:
                for resolve_include in app._include_resolved:
                    tar.add(resolve_include.src, arcname=resolve_include.dest)

            _, upload_native_url = self._union_remote.upload_file(Path(tar_path))

            return upload_native_url

    def materialize_values(self, app: App) -> dict:
        output = {}
        for user_input in app.inputs:
            if isinstance(user_input.value, ArtifactQuery):
                query = deepcopy(user_input.value)
                query.project = self.project
                query.domain = self.domain

                result = self._union_remote.get_artifact(query=query.to_flyte_idl())

                blob = result.literal.scalar.blob

                input_type = None
                if blob.metadata.type.dimensionality == BlobType.BlobDimensionality.SINGLE:
                    input_type = Input.Type.File
                elif blob.metadata.type.dimensionality == BlobType.BlobDimensionality.MULTIPART:
                    input_type = Input.Type.Directory

                output[user_input.name] = MaterializedInput(
                    value=result.literal.scalar.blob.uri,
                    type=input_type,
                )

            elif isinstance(user_input.value, URLQuery):
                query = user_input.value
                # Assume application has the same project and domain
                app_idl = self.get(name=query.name)
                if user_input.value.public:
                    output[user_input.name] = MaterializedInput(
                        value=app_idl.status.ingress.public_url,
                        input_type=Input.Type.String,
                    )
                else:
                    # TODO: User app_irl
                    app_id = app_idl.metadata.id
                    output[user_input.name] = MaterializedInput(
                        value=f"http://{app_id.name}.{app_id.project}-{app_id.domain}",
                        type=Input.Type.String,
                    )

        return output

    @property
    def sync_channel(self) -> grpc.Channel:
        try:
            return self._sync_channel
        except AttributeError:
            self._sync_channel = get_authenticated_channel(self.config.platform)
            return self._sync_channel

    @property
    def org(self) -> Optional[str]:
        try:
            return self._org
        except AttributeError:
            # TODO: Actually make _get_organization call
            self._org = self.config.platform.endpoint.split(".")[0]
            return self._org

    @property
    def sync_client(self) -> AppServiceStub:
        try:
            return self._sync_client
        except AttributeError:
            self._sync_client = AppServiceStub(self.sync_channel)
            return self._sync_client

    @property
    def async_client(self) -> AppServiceStub:
        try:
            return self._async_client
        except AttributeError:
            self._async_client = AppServiceStub(self.async_channel)
            return self._async_client

    @property
    def async_channel(self) -> grpc.aio.Channel:
        from union.filesystems._endpoint import _create_secure_channel_from_config

        try:
            return self._async_channel
        except AttributeError:
            self._async_channel = _create_secure_channel_from_config(self.config.platform, self.sync_channel)
            return self._async_channel

    @staticmethod
    def deployment_status(app_idl: AppIDL) -> str:
        try:
            current_status = app_idl.status.conditions[-1].deployment_status
            return Status.DeploymentStatus.Name(current_status).split("_")[-1].title()
        except Exception:
            return "Unknown"

    @staticmethod
    def desired_state(app_idl: AppIDL) -> str:
        return Spec.DesiredState.Name(app_idl.spec.desired_state).split("_")[-1].title()

    @staticmethod
    def get_limits(app_idl: AppIDL) -> dict:
        output = {}
        for limit in app_idl.spec.container.resources.limits:
            if limit.name == Resources.ResourceName.CPU:
                output["cpu"] = limit.value
            if limit.name == Resources.ResourceName.MEMORY:
                output["memory"] = limit.value
        return output



from datetime import datetime, timezone
from typing import Optional

import click
from click import secho, style
from flytekit.clis.sdk_in_container.constants import CTX_VERBOSE
from flytekit.models.task import Resources
from grpc import RpcError
from rich.console import Console
from rich.table import Table

from union._config import _get_config_obj
from union._utils import _timedelta_format
from union.cli._app_common import ENABLE_UNION_SERVING
from union.cli._common import _get_channel_with_org
from union.internal.common.list_pb2 import ListRequest
from union.internal.identity.app_payload_pb2 import ListAppsRequest
from union.internal.identity.app_service_pb2_grpc import AppsServiceStub
from union.internal.secret.payload_pb2 import ListSecretsRequest
from union.internal.secret.secret_pb2_grpc import SecretServiceStub
from union.remote import UnionRemote
from union.remote._app_remote import AppRemote
from union.workspace._vscode_remote import _get_vscode_link, _get_workspace_executions


@click.group()
def get():
    """Get a resource."""


@get.command()
@click.pass_context
@click.option("--project", help="Project name")
@click.option("--domain", help="Domain name")
def secret(
    ctx: click.Context,
    project: Optional[str],
    domain: Optional[str],
):
    """Get secrets."""
    platform_obj = _get_config_obj().platform
    channel, org = _get_channel_with_org(platform_obj)

    stub = SecretServiceStub(channel)
    secrets = []

    next_token, has_next = "", True

    try:
        while has_next:
            request = ListSecretsRequest(domain=domain, project=project, limit=20, token=next_token, organization=org)
            response = stub.ListSecrets(request)
            next_token = response.token
            has_next = next_token != ""

            secrets.extend(response.secrets)
    except RpcError as e:
        raise click.ClickException(f"Unable to get secrets.\n{e}") from e

    if secrets:
        table = Table()
        for name in ["name", "project", "domain"]:
            table.add_column(name, justify="right")

        for secret in secrets:
            project = secret.id.project or "-"
            domain = secret.id.domain or "-"
            table.add_row(secret.id.name, project, domain)

        console = Console()
        console.print(table)
    else:
        click.echo("No secrets found")


@get.command()
@click.pass_context
def app(ctx: click.Context):
    """Get applications."""
    platform_obj = _get_config_obj().platform
    channel, org = _get_channel_with_org(platform_obj)

    stub = AppsServiceStub(channel)

    apps = []
    next_token, has_next = "", True
    try:
        while has_next:
            request = ListAppsRequest(organization=org, request=ListRequest(limit=20, token=next_token))
            response = stub.List(request)
            next_token = response.token
            has_next = next_token != ""

            apps.extend(response.apps)
    except RpcError as e:
        raise click.ClickException(f"Unable to get apps.\n{e}") from e

    if apps:
        table = Table()
        table.add_column("client_id", overflow="fold")

        for app in apps:
            table.add_row(app.client_id)

        console = Console()
        console.print(table)
    else:
        click.echo("No apps found.")


@click.command()
@click.pass_context
@click.option("--name", default=None)
@click.option("--project", default="flytesnacks", help="Project name")
@click.option("--domain", default="development", help="Domain name")
def apps(ctx: click.Context, name: Optional[str], project: str, domain: str):
    """Get apps."""

    verbose_on = ctx.obj[CTX_VERBOSE] > 0
    app_remote = AppRemote(project=project, domain=domain)

    if name is None:
        # TODO: Add filter
        app_list = app_remote.list()
        if verbose_on:
            secho(app_list)
            return

        if not app_list:
            secho("No applications running")
            return

        table = Table()
        table.add_column("Name", overflow="fold")
        table.add_column("Link", overflow="fold")
        table.add_column("Status", overflow="fold")
        table.add_column("Desired State", overflow="fold")
        table.add_column("CPU", overflow="fold")
        table.add_column("Memory", overflow="fold")

        table_items = []

        for app in app_list:
            url = f"[link={app.status.ingress.public_url}]Click Here[/link]"
            limits = app_remote.get_limits(app)

            status = app_remote.deployment_status(app)
            desired_state = app_remote.desired_state(app)

            table_items.append(
                {
                    "name": app.metadata.id.name,
                    "url": url,
                    "status": status,
                    "desired_state": desired_state,
                    "cpu": limits.get("cpu", "-"),
                    "memory": limits.get("memory", "-"),
                }
            )

        sorted_table_items = sorted(table_items, key=lambda item: item["status"])

        for item in sorted_table_items:
            table.add_row(
                item["name"],
                item["url"],
                item["status"],
                item["desired_state"],
                item["cpu"],
                item["memory"],
            )

        console = Console()
        console.print(table)
    else:
        app_idl = app_remote.get(name=name)
        if verbose_on:
            secho(app_idl)
        else:
            limits = app_remote.get_limits(app_idl)
            details = {
                "name": app_idl.metadata.id.name,
                "status": app_remote.deployment_status(app_idl),
                "link": app_idl.status.ingress.public_url,
                "desired_state": app_remote.desired_state(app_idl),
                "cpu": limits.get("cpu", "-"),
                "memory": limits.get("memory", "-"),
            }
            items = ["name", "link", "status", "cpu", "memory", "desired_state"]
            max_item_len = max(len(i) for i in items)

            for item in items:
                styled_name = style(f"{item:<{max_item_len}}", bold=True)
                value = details[item]
                secho(f"{styled_name} : {value}")


if ENABLE_UNION_SERVING:
    get.add_command(apps)


@get.command()
@click.pass_context
@click.option("--name", default=None)
@click.option("--project", default="flytesnacks", help="Project name")
@click.option("--domain", default="development", help="Domain name")
@click.option("--show-details", default=False, is_flag=True, help="Show additional details")
def workspace(ctx: click.Context, name: Optional[str], project: str, domain: str, show_details: bool):
    """Get workspaces."""
    remote = UnionRemote(default_domain=domain, default_project=project)

    if name is None:
        _list_workspaces(remote, project, domain)
    else:
        _show_single_workspace(remote, name, project, domain, show_details)


def _show_single_workspace(remote: UnionRemote, name: str, project: str, domain: str, show_details: bool):
    executions = _get_workspace_executions(remote, project, domain)

    named_execution = None
    for execution, ex_name in executions:
        if ex_name == name:
            named_execution = execution
            break
    else:  # no break
        raise click.ClickException(f"Unable to find workspace with name: {name}")

    details = _get_details(remote, name, named_execution)
    items = ["name", "cpu", "memory", "duration"]

    if show_details:
        items.append("execution_url")

    items.append("workspace_url")

    max_item_len = max(len(i) for i in items)

    for item in items:
        styled_name = style(f"{item:<{max_item_len}}", bold=True)
        value = details[item]
        secho(f"{styled_name} : {value}")


def _get_details(remote: UnionRemote, name: str, execution) -> dict:
    output = {"name": name, "cpu": "default", "memory": "default", "gpu": "none"}

    launch_plan = execution.spec.launch_plan
    task = remote.fetch_task(
        project=launch_plan.project,
        domain=launch_plan.domain,
        name=launch_plan.name,
        version=launch_plan.version,
    )

    # get resources information
    requests = task.template.container.resources.requests
    for request in requests:
        if request.name == Resources.ResourceName.CPU:
            output["cpu"] = request.value
        elif request.name == Resources.ResourceName.MEMORY:
            output["memory"] = request.value
        elif request.name == Resources.ResourceName.GPU:
            # TODO: Fix with correct metadata
            output["gpu"] = "1"

    output["version"] = launch_plan.version
    output["workspace_url"] = _get_vscode_link(remote, execution)

    workflow_execution = remote.fetch_execution(
        project=execution.id.project,
        domain=execution.id.domain,
        name=execution.id.name,
    )
    output["execution_url"] = remote.generate_console_url(workflow_execution)

    now = datetime.now(tz=timezone.utc)
    duration = now - execution.closure.started_at
    output["duration"] = _timedelta_format(duration)

    return output


def _list_workspaces(remote: UnionRemote, project: str, domain: str):
    executions = _get_workspace_executions(remote, project, domain)

    if not executions:
        click.echo("No workspaces found.")
        return

    table = Table()
    table.add_column("Name", overflow="fold")
    table.add_column("CPU", overflow="fold")
    table.add_column("Memory", overflow="fold")
    table.add_column("GPU", overflow="fold")
    table.add_column("Duration", overflow="fold")
    table.add_column("Link", overflow="fold")

    for execution, name in executions:
        details = _get_details(remote, name, execution)
        if details["workspace_url"] == "Unavailable":
            url = "Unavailable"
        else:
            url = f"[link={details['workspace_url']}]Click Here[/link]"

        table.add_row(
            name,
            details["cpu"],
            details["memory"],
            details["gpu"],
            details["duration"],
            url,
        )

    console = Console()
    console.print(table)



import logging
import os
import typing

from flyteidl.core import artifact_id_pb2 as art_id
from flytekit import BlobType
from flytekit.clients.friendly import SynchronousFlyteClient
from flytekit.configuration import Config
from flytekit.core.type_engine import LiteralsResolver, TypeEngine
from flytekit.exceptions import user as user_exceptions
from flytekit.models.literals import Blob, BlobMetadata, Literal, Scalar
from flytekit.remote import FlyteRemote
from flytekit.remote.entities import FlyteLaunchPlan, FlyteTask, FlyteWorkflow
from flytekit.remote.executions import FlyteNodeExecution, FlyteTaskExecution, FlyteWorkflowExecution
from flytekit.tools.translator import (
    Options,
)

from union._config import ConfigSource, ConfigWithSource, _get_config_obj, _get_organization
from union._interceptor import update_client_with_interceptor
from union.artifacts import Artifact
from union.internal.artifacts import artifacts_pb2, artifacts_pb2_grpc

SERVERLESS_VANITY_URLS = {
    "https://serverless-1.us-east-2.s.union.ai": "https://serverless.union.ai",
    "https://serverless-preview.canary.unionai.cloud": "https://serverless.canary.union.ai",
    "https://serverless-gcp.cloud-staging.union.ai": "https://serverless.staging.union.ai",
}


class UnionRemote(FlyteRemote):
    def __init__(
        self,
        config: typing.Optional[Config] = None,
        default_project: typing.Optional[str] = "flytesnacks",
        default_domain: typing.Optional[str] = "development",
        data_upload_location: str = "flyte://my-s3-bucket/",
        **kwargs,
    ):
        import union._config
        from union.ucimage._image_builder import _register_union_image_builder

        if config is None:
            config = _get_config_obj(config, default_to_union_semantics=True)

        config_with_source = ConfigWithSource(config=config, source=ConfigSource.REMOTE)
        config = _get_config_obj(config_with_source, default_to_union_semantics=True)

        # register Union image builder when getting remote so it's available for
        # jupyter notebook task and workflow execution.
        if union._config._should_enable_image_builder(config.platform.endpoint):
            _register_union_image_builder()

        super().__init__(config, default_project, default_domain, data_upload_location, **kwargs)
        self._artifacts_client = None

    @property
    def client(self) -> SynchronousFlyteClient:
        """Return a SynchronousFlyteClient for additional operations."""
        if not self._client_initialized:
            client = SynchronousFlyteClient(self.config.platform, **self._kwargs)
            org = _get_organization(self.config.platform, channel=client._channel)
            self._client = update_client_with_interceptor(client, org)
            self._client_initialized = True

        return self._client

    def generate_console_http_domain(self) -> str:
        default_console_http_domain = super().generate_console_http_domain()
        return SERVERLESS_VANITY_URLS.get(default_console_http_domain, default_console_http_domain)

    def generate_console_url(
        self,
        entity: typing.Union[
            FlyteWorkflowExecution, FlyteNodeExecution, FlyteTaskExecution, FlyteWorkflow, FlyteTask, FlyteLaunchPlan
        ],
    ):
        """
        Generate a UnionAI console URL for the given Flyte remote endpoint.

        This will automatically determine if this is an execution or an entity
        and change the type automatically.
        """
        org = _get_organization(self.config.platform)
        url = super().generate_console_url(entity)
        if org is None:
            return url

        console_http_domain = self.generate_console_http_domain()
        old_prefix = f"{console_http_domain}/console/"
        new_prefix = f"{console_http_domain}/org/{org}/"

        return url.replace(old_prefix, new_prefix)

    @property
    def artifacts_client(self) -> artifacts_pb2_grpc.ArtifactRegistryStub:
        if self._artifacts_client:
            return self._artifacts_client

        self._artifacts_client = artifacts_pb2_grpc.ArtifactRegistryStub(self.client._channel)
        return self._artifacts_client

    def get_artifact(
        self,
        uri: typing.Optional[str] = None,
        artifact_key: typing.Optional[art_id.ArtifactKey] = None,
        artifact_id: typing.Optional[art_id.ArtifactID] = None,
        query: typing.Optional[art_id.ArtifactQuery] = None,
        get_details: bool = False,
    ) -> typing.Optional[Artifact]:
        """
        Get the specified artifact.

        :param uri: An artifact URI.
        :param artifact_key: An artifact key.
        :param artifact_id: The artifact ID.
        :param query: An artifact query.
        :param get_details: A bool to indicate whether or not to return artifact details.
        :return: The artifact as persisted in the service.
        """
        if query:
            q = query
        elif uri:
            q = art_id.ArtifactQuery(uri=uri)
        elif artifact_key:
            q = art_id.ArtifactQuery(artifact_id=art_id.ArtifactID(artifact_key=artifact_key))
        elif artifact_id:
            q = art_id.ArtifactQuery(artifact_id=artifact_id)
        else:
            raise ValueError("One of uri, key, id")
        req = artifacts_pb2.GetArtifactRequest(query=q, details=get_details)
        resp = self.artifacts_client.GetArtifact(req)
        a = Artifact.from_flyte_idl(resp.artifact)
        if a.literal and a.name:
            # assigned here because the resolver has an implicit dependency on the remote's context
            # can move into artifact if we are okay just using the current context.
            a.resolver = LiteralsResolver(literals={a.name: a.literal}, variable_map=None, ctx=self.context)
        return a

    def _execute(
        self,
        entity: typing.Union[FlyteTask, FlyteWorkflow, FlyteLaunchPlan],
        inputs: typing.Dict[str, typing.Any],
        project: str = None,
        domain: str = None,
        execution_name: typing.Optional[str] = None,
        execution_name_prefix: typing.Optional[str] = None,
        options: typing.Optional[Options] = None,
        wait: bool = False,
        type_hints: typing.Optional[typing.Dict[str, typing.Type]] = None,
        overwrite_cache: typing.Optional[bool] = None,
        envs: typing.Optional[typing.Dict[str, str]] = None,
        tags: typing.Optional[typing.List[str]] = None,
        cluster_pool: typing.Optional[str] = None,
        **kwargs,
    ) -> FlyteWorkflowExecution:
        resolved_inputs = {}
        for k, v in inputs.items():
            # TODO: Remove when https://github.com/flyteorg/flytekit/pull/2136 gets merged
            if Artifact is None:
                resolved_inputs[k] = v
            elif isinstance(v, artifacts_pb2.Artifact):
                lit = v.spec.value
                resolved_inputs[k] = lit
            elif isinstance(v, Artifact):
                if v.literal is not None:
                    lit = v.literal
                elif v.to_id_idl() is not None:
                    fetched_artifact = self.get_artifact(artifact_id=v.concrete_artifact_id)
                    if not fetched_artifact:
                        raise user_exceptions.FlyteValueException(
                            v.concrete_artifact_id, "Could not find artifact with ID given"
                        )
                    lit = fetched_artifact.literal
                else:
                    raise user_exceptions.FlyteValueException(
                        v, "When binding input to Artifact, either the Literal or the ID must be set"
                    )
                resolved_inputs[k] = lit
            else:
                resolved_inputs[k] = v

        return super()._execute(
            entity,
            resolved_inputs,
            project=project,
            domain=domain,
            execution_name=execution_name,
            execution_name_prefix=execution_name_prefix,
            options=options,
            wait=wait,
            type_hints=type_hints,
            overwrite_cache=overwrite_cache,
            envs=envs,
            tags=tags,
            cluster_pool=cluster_pool,
            **kwargs,
        )

    def create_artifact(self, artifact: Artifact):
        """
        Create an artifact in FlyteAdmin.

        :param artifact: The artifact to create.
        :return: The artifact as persisted in the service.
        """
        # Two things can happen here -
        #  - the call to to_literal may upload something, in the case of an offloaded data type.
        #    - if this happens, the upload request should already return the created Artifact object.
        if artifact.literal is None:
            with self.remote_context() as ctx:
                lt = artifact.literal_type or TypeEngine.to_literal_type(artifact.python_type)
                lit = TypeEngine.to_literal(ctx, artifact.python_val, artifact.python_type, lt)
                artifact.literal_type = lt.to_flyte_idl()
                artifact.literal = lit.to_flyte_idl()
        else:
            raise ValueError("Cannot create an artifact with a literal already set.")

        if artifact.project is None:
            artifact.project = self.default_project
        if artifact.domain is None:
            artifact.domain = self.default_domain
        create_request = Artifact.to_create_request(artifact)
        logging.debug(f"CreateArtifact request {create_request}")
        resp = self.artifacts_client.CreateArtifact(create_request)
        logging.debug(f"CreateArtifact response {resp}")

    def _has_file_extension(self, file_path) -> bool:
        _, ext = os.path.splitext(file_path)
        return bool(ext)

    def get(self, uri: typing.Optional[str] = None) -> typing.Optional[typing.Union[LiteralsResolver, Literal, bytes]]:
        if not uri.startswith("union://"):
            # Assume this is the default behavior and this function is being called with a flyte uri
            return super().get(uri)
        if self._has_file_extension(uri):
            blob_dimensionality = BlobType.BlobDimensionality.SINGLE
        else:
            blob_dimensionality = BlobType.BlobDimensionality.MULTIPART
        return Literal(
            scalar=Scalar(
                blob=Blob(
                    metadata=BlobMetadata(
                        type=BlobType(
                            format="",
                            dimensionality=blob_dimensionality,
                        ),
                    ),
                    uri=uri,
                )
            )
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: app/app_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.app import app_payload_pb2 as app_dot_app__payload__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x61pp/app_service.proto\x12\x0c\x63loudidl.app\x1a\x15\x61pp/app_payload.proto\x1a\x1cgoogle/api/annotations.proto2\xdf\n\n\nAppService\x12[\n\x06\x43reate\x12\x1b.cloudidl.app.CreateRequest\x1a\x1c.cloudidl.app.CreateResponse\"\x16\x82\xd3\xe4\x93\x02\x10\"\x0b/app/api/v1:\x01*\x12\xce\x01\n\x03Get\x12\x18.cloudidl.app.GetRequest\x1a\x19.cloudidl.app.GetResponse\"\x91\x01\x90\x02\x01\x82\xd3\xe4\x93\x02\x87\x01\x12:/app/api/v1/{app_id.project}/{app_id.domain}/{app_id.name}ZI\x12G/app/api/v1/{app_id.org}/{app_id.project}/{app_id.domain}/{app_id.name}\x12\x99\x02\n\x06Update\x12\x1b.cloudidl.app.UpdateRequest\x1a\x1c.cloudidl.app.UpdateResponse\"\xd3\x01\x82\xd3\xe4\x93\x02\xcc\x01\x1aU/app/api/v1/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\x01*Zp\x1ak/app/api/v1/{app.metadata.id.org}/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\x01*\x12\xb9\x02\n\x0cUpdateStatus\x12!.cloudidl.app.UpdateStatusRequest\x1a\".cloudidl.app.UpdateStatusResponse\"\xe1\x01\x82\xd3\xe4\x93\x02\xda\x01\x1a\\/app/api/v1/status/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\x01*Zw\x1ar/app/api/v1/status/{app.metadata.id.org}/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\x01*\x12\xd4\x01\n\x06\x44\x65lete\x12\x1b.cloudidl.app.DeleteRequest\x1a\x1c.cloudidl.app.DeleteResponse\"\x8e\x01\x82\xd3\xe4\x93\x02\x87\x01*:/app/api/v1/{app_id.project}/{app_id.domain}/{app_id.name}ZI*G/app/api/v1/{app_id.org}/{app_id.project}/{app_id.domain}/{app_id.name}\x12\x65\n\x04List\x12\x19.cloudidl.app.ListRequest\x1a\x1a.cloudidl.app.ListResponse\"&\x90\x02\x01\x82\xd3\xe4\x93\x02\x1d\x12\x0b/app/api/v1Z\x0e\x12\x0c/apps/api/v1\x12G\n\x05Watch\x12\x1a.cloudidl.app.WatchRequest\x1a\x1b.cloudidl.app.WatchResponse\"\x03\x90\x02\x01\x30\x01\x12\x44\n\x05Lease\x12\x1a.cloudidl.app.LeaseRequest\x1a\x1b.cloudidl.app.LeaseResponse\"\x00\x30\x01\x42\x9e\x01\n\x10\x63om.cloudidl.appB\x0f\x41ppServiceProtoH\x02P\x01Z&github.com/unionai/cloud/gen/pb-go/app\xa2\x02\x03\x43\x41X\xaa\x02\x0c\x43loudidl.App\xca\x02\x0c\x43loudidl\\App\xe2\x02\x18\x43loudidl\\App\\GPBMetadata\xea\x02\rCloudidl::Appb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'app.app_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\020com.cloudidl.appB\017AppServiceProtoH\002P\001Z&github.com/unionai/cloud/gen/pb-go/app\242\002\003CAX\252\002\014Cloudidl.App\312\002\014Cloudidl\\App\342\002\030Cloudidl\\App\\GPBMetadata\352\002\rCloudidl::App'
  _APPSERVICE.methods_by_name['Create']._options = None
  _APPSERVICE.methods_by_name['Create']._serialized_options = b'\202\323\344\223\002\020\"\013/app/api/v1:\001*'
  _APPSERVICE.methods_by_name['Get']._options = None
  _APPSERVICE.methods_by_name['Get']._serialized_options = b'\220\002\001\202\323\344\223\002\207\001\022:/app/api/v1/{app_id.project}/{app_id.domain}/{app_id.name}ZI\022G/app/api/v1/{app_id.org}/{app_id.project}/{app_id.domain}/{app_id.name}'
  _APPSERVICE.methods_by_name['Update']._options = None
  _APPSERVICE.methods_by_name['Update']._serialized_options = b'\202\323\344\223\002\314\001\032U/app/api/v1/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\001*Zp\032k/app/api/v1/{app.metadata.id.org}/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\001*'
  _APPSERVICE.methods_by_name['UpdateStatus']._options = None
  _APPSERVICE.methods_by_name['UpdateStatus']._serialized_options = b'\202\323\344\223\002\332\001\032\\/app/api/v1/status/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\001*Zw\032r/app/api/v1/status/{app.metadata.id.org}/{app.metadata.id.project}/{app.metadata.id.domain}/{app.metadata.id.name}:\001*'
  _APPSERVICE.methods_by_name['Delete']._options = None
  _APPSERVICE.methods_by_name['Delete']._serialized_options = b'\202\323\344\223\002\207\001*:/app/api/v1/{app_id.project}/{app_id.domain}/{app_id.name}ZI*G/app/api/v1/{app_id.org}/{app_id.project}/{app_id.domain}/{app_id.name}'
  _APPSERVICE.methods_by_name['List']._options = None
  _APPSERVICE.methods_by_name['List']._serialized_options = b'\220\002\001\202\323\344\223\002\035\022\013/app/api/v1Z\016\022\014/apps/api/v1'
  _APPSERVICE.methods_by_name['Watch']._options = None
  _APPSERVICE.methods_by_name['Watch']._serialized_options = b'\220\002\001'
  _globals['_APPSERVICE']._serialized_start=93
  _globals['_APPSERVICE']._serialized_end=1468
# @@protoc_insertion_point(module_scope)
